@@ -45,11 +45,11 @@ static void gvir_config_domain_interface_bridge_class_init(GVirConfigDomainInter
 }
 
 
-static void gvir_config_domain_interface_bridge_init(GVirConfigDomainInterfaceBridge *conn)
+static void gvir_config_domain_interface_bridge_init(GVirConfigDomainInterfaceBridge *bridge)
 {
-    g_debug("Init GVirConfigDomainInterfaceBridge=%p", conn);
+    g_debug("Init GVirConfigDomainInterfaceBridge=%p", bridge);
 
-    conn->priv = GVIR_CONFIG_DOMAIN_INTERFACE_BRIDGE_GET_PRIVATE(conn);
+    bridge->priv = GVIR_CONFIG_DOMAIN_INTERFACE_BRIDGE_GET_PRIVATE(bridge);
 }
 
 
