@@ -111,6 +111,9 @@ int main(int argc, char **argv)
 
     domain = gvir_config_domain_new();
     g_assert(domain != NULL);
+
+    gvir_config_domain_set_virt_type(domain, GVIR_CONFIG_DOMAIN_VIRT_KVM);
+    g_assert(gvir_config_domain_get_virt_type(domain) == GVIR_CONFIG_DOMAIN_VIRT_KVM);
     gvir_config_domain_set_name(domain, "foo");
     g_str_const_check(gvir_config_domain_get_name(domain), "foo");
 
@@ -135,15 +138,20 @@ int main(int argc, char **argv)
     GVirConfigDomainClock *klock;
     GVirConfigDomainTimerPit *pit;
     GVirConfigDomainTimerRtc *rtc;
+    GVirConfigDomainTimerHpet *hpet;
 
     klock = gvir_config_domain_clock_new();
-    gvir_config_domain_clock_set_offset(klock, GVIR_CONFIG_DOMAIN_CLOCK_UTC);
+    gvir_config_domain_clock_set_offset(klock, GVIR_CONFIG_DOMAIN_CLOCK_TIMEZONE);
+    gvir_config_domain_clock_set_timezone(klock, "CEST");
+    g_assert(gvir_config_domain_clock_get_offset(klock) == GVIR_CONFIG_DOMAIN_CLOCK_TIMEZONE);
+    g_str_const_check(gvir_config_domain_clock_get_timezone(klock), "CEST");
 
     pit = gvir_config_domain_timer_pit_new();
     gvir_config_domain_timer_set_tick_policy(GVIR_CONFIG_DOMAIN_TIMER(pit),
                                              GVIR_CONFIG_DOMAIN_TIMER_TICK_POLICY_DELAY);
     gvir_config_domain_clock_add_timer(klock, GVIR_CONFIG_DOMAIN_TIMER(pit));
     g_assert(gvir_config_domain_timer_get_tick_policy(GVIR_CONFIG_DOMAIN_TIMER(pit)) == GVIR_CONFIG_DOMAIN_TIMER_TICK_POLICY_DELAY);
+    g_assert(gvir_config_domain_timer_get_present(GVIR_CONFIG_DOMAIN_TIMER(pit)) != FALSE);
     g_object_unref(G_OBJECT(pit));
 
     rtc = gvir_config_domain_timer_rtc_new();
@@ -151,11 +159,24 @@ int main(int argc, char **argv)
                                              GVIR_CONFIG_DOMAIN_TIMER_TICK_POLICY_CATCHUP);
     gvir_config_domain_clock_add_timer(klock, GVIR_CONFIG_DOMAIN_TIMER(rtc));
     g_assert(gvir_config_domain_timer_get_tick_policy(GVIR_CONFIG_DOMAIN_TIMER(rtc)) == GVIR_CONFIG_DOMAIN_TIMER_TICK_POLICY_CATCHUP);
+    g_assert(gvir_config_domain_timer_get_present(GVIR_CONFIG_DOMAIN_TIMER(rtc)) != FALSE);
     g_object_unref(G_OBJECT(rtc));
 
+    hpet = gvir_config_domain_timer_hpet_new();
+    gvir_config_domain_timer_set_present(GVIR_CONFIG_DOMAIN_TIMER(hpet), FALSE);
+    gvir_config_domain_clock_add_timer(klock, GVIR_CONFIG_DOMAIN_TIMER(hpet));
+    g_assert(gvir_config_domain_timer_get_present(GVIR_CONFIG_DOMAIN_TIMER(hpet)) == FALSE);
+    g_object_unref(G_OBJECT(hpet));
+
     gvir_config_domain_set_clock(domain, klock);
     g_object_unref(G_OBJECT(klock));
 
+    klock = gvir_config_domain_get_clock(domain);
+    g_assert(klock != NULL);
+    g_assert(gvir_config_domain_clock_get_offset(klock) == GVIR_CONFIG_DOMAIN_CLOCK_TIMEZONE);
+    g_str_const_check(gvir_config_domain_clock_get_timezone(klock), "CEST");
+    g_object_unref(G_OBJECT(klock));
+
     /* os node */
     GVirConfigDomainOs *os;
     GList *devices = NULL;
@@ -230,28 +251,37 @@ int main(int argc, char **argv)
 
     /* disk node */
     GVirConfigDomainDisk *disk;
+    GVirConfigDomainDiskDriver *driver;
+
+    driver = gvir_config_domain_disk_driver_new();
+    gvir_config_domain_disk_driver_set_name(driver, "foo");
+    gvir_config_domain_disk_driver_set_format(driver, GVIR_CONFIG_DOMAIN_DISK_FORMAT_BOCHS);
+    gvir_config_domain_disk_driver_set_name(driver, "qemu");
+    gvir_config_domain_disk_driver_set_cache(driver, GVIR_CONFIG_DOMAIN_DISK_CACHE_NONE);
+    gvir_config_domain_disk_driver_set_format(driver, GVIR_CONFIG_DOMAIN_DISK_FORMAT_QCOW2);
+    gvir_config_domain_disk_driver_set_copy_on_read(driver, TRUE);
 
     disk = gvir_config_domain_disk_new();
     gvir_config_domain_disk_set_type(disk, GVIR_CONFIG_DOMAIN_DISK_FILE);
     gvir_config_domain_disk_set_guest_device_type(disk, GVIR_CONFIG_DOMAIN_DISK_GUEST_DEVICE_DISK);
     gvir_config_domain_disk_set_source(disk, "/tmp/foo/bar");
     gvir_config_domain_disk_set_startup_policy (disk, GVIR_CONFIG_DOMAIN_DISK_STARTUP_POLICY_REQUISITE);
-    gvir_config_domain_disk_set_driver_name(disk, "foo");
-    gvir_config_domain_disk_set_driver_type(disk, "bar");
-    gvir_config_domain_disk_set_driver_name(disk, "qemu");
-    gvir_config_domain_disk_set_driver_cache(disk, GVIR_CONFIG_DOMAIN_DISK_CACHE_NONE);
-    gvir_config_domain_disk_set_driver_type(disk, "qcow2");
     gvir_config_domain_disk_set_target_bus(disk, GVIR_CONFIG_DOMAIN_DISK_BUS_IDE);
     gvir_config_domain_disk_set_target_dev(disk, "hda");
+    gvir_config_domain_disk_set_driver(disk, driver);
+    g_object_unref(G_OBJECT(driver));
     devices = g_list_append(devices, GVIR_CONFIG_DOMAIN_DEVICE(disk));
 
     g_assert(gvir_config_domain_disk_get_disk_type(disk) == GVIR_CONFIG_DOMAIN_DISK_FILE);
     g_assert(gvir_config_domain_disk_get_guest_device_type(disk) == GVIR_CONFIG_DOMAIN_DISK_GUEST_DEVICE_DISK);
     g_assert(gvir_config_domain_disk_get_startup_policy (disk) == GVIR_CONFIG_DOMAIN_DISK_STARTUP_POLICY_REQUISITE);
     g_str_const_check(gvir_config_domain_disk_get_source(disk), "/tmp/foo/bar");
-    g_assert(gvir_config_domain_disk_get_driver_cache(disk) == GVIR_CONFIG_DOMAIN_DISK_CACHE_NONE);
-    g_str_const_check(gvir_config_domain_disk_get_driver_name(disk), "qemu");
-    g_str_const_check(gvir_config_domain_disk_get_driver_type(disk), "qcow2");
+    driver = gvir_config_domain_disk_get_driver(disk);
+    g_assert(driver != NULL);
+    g_assert(gvir_config_domain_disk_driver_get_cache(driver) == GVIR_CONFIG_DOMAIN_DISK_CACHE_NONE);
+    g_str_const_check(gvir_config_domain_disk_driver_get_name(driver), "qemu");
+    g_assert(gvir_config_domain_disk_driver_get_format(driver) == GVIR_CONFIG_DOMAIN_DISK_FORMAT_QCOW2);
+    g_assert(gvir_config_domain_disk_driver_get_copy_on_read(driver));
     g_assert(gvir_config_domain_disk_get_target_bus(disk) == GVIR_CONFIG_DOMAIN_DISK_BUS_IDE);
     g_str_const_check(gvir_config_domain_disk_get_target_dev(disk), "hda");
 
@@ -402,6 +432,9 @@ int main(int argc, char **argv)
     gvir_config_storage_pool_source_set_directory(pool_source, "/foo/bar");
     gvir_config_storage_pool_set_source(pool, pool_source);
     g_object_unref(G_OBJECT(pool_source));
+    pool_source = gvir_config_storage_pool_get_source(pool);
+    g_str_const_check(gvir_config_storage_pool_source_get_directory(pool_source), "/foo/bar");
+    g_object_unref(G_OBJECT(pool_source));
 
     perms = gvir_config_storage_permissions_new();
     gvir_config_storage_permissions_set_owner(perms, 1001);
@@ -416,6 +449,16 @@ int main(int argc, char **argv)
     gvir_config_storage_pool_set_target(pool, pool_target);
     g_object_unref(G_OBJECT(pool_target));
 
+    pool_target = gvir_config_storage_pool_get_target(pool);
+    g_str_const_check(gvir_config_storage_pool_target_get_path(pool_target), "/dev/disk/by-path");
+    perms = gvir_config_storage_pool_target_get_permissions(pool_target);
+    g_object_unref(G_OBJECT(pool_target));
+    g_assert(gvir_config_storage_permissions_get_owner(perms) == 1001);
+    g_assert(gvir_config_storage_permissions_get_group(perms) == 1007);
+    g_assert(gvir_config_storage_permissions_get_mode(perms) == 0744);
+    g_str_const_check(gvir_config_storage_permissions_get_label(perms), "virt_image_t");
+    g_object_unref(G_OBJECT(perms));
+
     xml = gvir_config_object_to_xml(GVIR_CONFIG_OBJECT(pool));
     g_print("%s\n\n", xml);
     g_free(xml);
